import datetime
import signal
import asyncio
import sys
from ovtp.server import OvtpServer
from ovtp.server.add_auth_keys import add_auth_keys


def signal_handler(sig, frame):
    print("Pressed Ctrl+C, exiting...")
    exit(0)


def callback(status, response):
    print(f'status: {status}, response: {response}')
    return 'OK', 'GOOD'


def run_server():
    print("Running script %s" % datetime.datetime.now())
    signal.signal(signal.SIGINT, signal_handler)
    print('Press Ctrl+C for exit')
    ovtp_server = OvtpServer(callback, verbose=False, debug=False)
    if len(sys.argv) > 2:
        if sys.argv[2] == '--daemon':
            print('Running in daemon mode')
        elif sys.argv[2] == '--add-keys':
            print('Adding new auth keys...')
            add_auth_keys()
            sys.exit(0)
    else:
        print('Running in normal mode. To run in daemon mode, run with --daemon')
        main_mode = input('Select mode d, v, dv - debug, verbose: ')
        if main_mode in ['d', 'v', 'dv']:
            if main_mode in ['d', 'dv']:
                ovtp_server.debug = True
            if main_mode in ['v', 'dv']:
                ovtp_server.verbose = True
    asyncio.run(ovtp_server.run_server())



import os
import rsa
import pathlib
import ovcrypt
import asyncio
import oe_common
import ov_aes_cipher
import json
from ovtp.client import cfg


script_run = True


class OvtpClient:
    def __init__(self, server_ip, server_port=None, debug=False, verbose=False):
        self.uid = oe_common.get_rnd_string(10).lower()
        self.cr = ovcrypt.OvCrypt()
        self.reader = None
        self.writer = None
        self.server_public_key = None
        self.encrypted_connection = False
        self.server_address = server_ip
        if server_port:
            self.server_port = server_port
        else:
            self.server_port = cfg['default_server_port']
        self.debug = debug
        self.verbose = verbose
        self.authorized = False
        self.connected = False
        self.aes = None
        self.loop = asyncio.new_event_loop()

    async def connect(self):
        self.reader, self.writer = await asyncio.open_connection(
            self.server_address, self.server_port
        )
        self.connected = True

    async def reconnect(self):
        self.reset()
        await self.check_connection()

    async def close_connection(self):
        try:
            if self.verbose:
                print(f'Closing connection with {self.server_address}')
            self.writer.write_eof()
            await self.writer.drain()
            self.writer.close()
            await self.writer.wait_closed()
        except OSError as e:
            print(f'OSError occurred: "{e}"')
        finally:
            self.reset()

    def reset(self):
        self.reader = None
        self.writer = None
        self.encrypted_connection = False
        self.authorized = False
        self.connected = False
        self.server_public_key = None

    async def _send_receive_keys(self):
        if self.server_public_key is None:
            server_key_bytes = await self.send_data(
                self.server_address,
                data=rsa.PublicKey.save_pkcs1(self.cr.public_key),
                data_type='public_key'
            )
            if len(server_key_bytes) < 10:
                raise RuntimeError("Received public key is invalid!")
            try:
                self.server_public_key = rsa.PublicKey.load_pkcs1(server_key_bytes)
                self.encrypted_connection = True
                if self.debug:
                    print('Connection encrypted')
            except ValueError as e:
                if self.verbose:
                    print('Received public key is not valid:', e)
                raise
                # return False
        return True

    async def _authorize(self):
        auth_str_request = await self.send_data(
            self.server_address,
            data=oe_common.get_rnd_string().encode(),
            data_type='auth_req'
        )
        auth_str = rsa.decrypt(auth_str_request, self.cr.private_key)
        if self.debug:
            print(f'Auth str: {auth_str}')
        auth_response = await self.send_data(
            self.server_address,
            data=auth_str,
            data_type='auth_resp'
        )
        if self.debug:
            print(f'Auth response: {auth_response}')
        if auth_response == 'auth successful':
            if self.debug:
                print(f'Auth successful')
            return True
        if self.debug:
            print(f'Auth failed')
        return False

    @staticmethod
    def _pad_ov_header(bs, s):
        if len(s) % bs != 0:
            length = bs - (len(s) % bs)
            s += bytes(b'\x00') * length
            # print(f'len is {length}')
        return s

    @staticmethod
    def _pack_data(d):
        if d:
            # enc = d.encode()
            return b''.join([
                len(d).to_bytes(4, byteorder='big'),
                d
            ])
        else:
            return (0).to_bytes(4, byteorder='big')

    def _get_ov_header(self, data_type, filename=None, key=None, iv=None, signed=None, sign=None, size=None):
        ba = [
            self._pad_ov_header(10, data_type.encode()),
            self._pack_data(filename.encode() if filename else None),
            self._pack_data(key),
            self._pack_data(iv),
            b'y' if signed else b'n',
            # self._pack_data(sign) if signed else b'',
            int(size).to_bytes(14, byteorder='big') if data_type == 'file' else b''
        ]
        return b''.join(ba)

    @staticmethod
    def _validate_input(data_type, filename):
        error = 'Unknown error'
        if data_type == 'file':
            if os.path.isfile(filename):
                return True
            else:
                error = f"File {filename} does not exists!"
        raise RuntimeError(error)

    async def read_with_prefix(self, timeout=5, retries=0):
        prefix_timeout = 30
        if timeout > prefix_timeout:
            prefix_timeout = timeout
        try:
            prefix = (await asyncio.wait_for(self.reader.readline(), timeout=prefix_timeout)).rstrip()
        except asyncio.exceptions.TimeoutError:
            if retries:
                print(f"Timeout error, but {retries} retries left")
                return await self.read_with_prefix(timeout=timeout, retries=retries-1)
            else:
                print(f"Timeout error, no retries left")
                raise
        pk_data = b''
        msg_len = 0
        if prefix != b'':
            msg_len = int(prefix[:-3])
            pk_data = await asyncio.wait_for(self.reader.readexactly(msg_len), timeout=timeout)
        return pk_data, msg_len

    async def write_with_prefix(self, data, prefix=False):
        if prefix:
            prefix_pad = b'pad'
        else:
            prefix_pad = b'\x00' * 3
        self.writer.write(b'%d%b\n%b' % (len(data), prefix_pad, data))
        await self.writer.drain()

    async def check_connection(self):
        if not self.connected:
            await self.connect()
        if await self._send_receive_keys():
            if not self.authorized:
                if await self._authorize():
                    self.authorized = True
                else:
                    raise RuntimeError('Auth failed')

    async def verify_file_sign(self, path):
        await self.check_connection()
        self.writer.write(self._get_ov_header(
            'get_hash',
            filename=path
        ))
        await self.writer.drain()
        file_hash_encoded, length = await self.read_with_prefix()
        file_hash = rsa.decrypt(file_hash_encoded, self.cr.private_key)
        file_sign = rsa.sign(file_hash, self.cr.private_key, 'SHA-1')
        if self.debug:
            print(f'File hash: {file_hash}')
            print(f'File sign: {file_sign}')
        if file_hash == b'file_not_exists':
            return False
        else:
            # local_sign_location = os.path.join(pathlib.Path().absolute(), cfg['signs_dir'], self.cr.get_ip_hash(self.server_address))
            return file_sign

    async def get_file(self, path_from, path_to=None):
        if not path_to:
            path_to = path_from
        await self.check_connection()
        self.aes = ov_aes_cipher.AESCipher(key=oe_common.get_rnd_string(60))
        self.writer.write(self._get_ov_header(
            'get_file',
            filename=path_from,
            key=rsa.encrypt(self.aes.key, self.server_public_key),
            iv=rsa.encrypt(self.aes.iv, self.server_public_key),
            signed=True
        ))
        # print(f'zero: {path_to[0]}')
        if path_to[0] != '/':
            path_to = os.path.join(pathlib.Path().absolute(), path_to)
        oe_common.check_create_dir(path_to)
        await self.writer.drain()
        ov_sign = ovcrypt.OvSign(self.server_public_key)
        dc = oe_common.DinConsole()
        sc = oe_common.SpeedChecker('bits / second', 4096)
        rx_bytes = 0
        with open(path_to, 'wb') as f:
            while True:
                part, length = (await self.read_with_prefix())

                rx_bytes += length
                if self.debug or self.verbose:
                    dc.update('  Received %s / %s, %s%%, speed: %s' % (
                        oe_common.convert_size(rx_bytes),
                        0,
                        '?',
                        sc.get_speed()
                    ))

                if length == 256:
                    if part.rstrip(b'\x00') == b'ovsign':
                        received_sign = await self.reader.read(256)
                        if self.debug or self.verbose:
                            dc.stay()
                        if ov_sign.get_verification_result(received_sign):
                            if self.verbose:
                                print(f'File downloaded to: {path_to}')
                            return True
                        else:
                            if self.verbose:
                                print(f'Error downloading file: {path_to}')
                            return False
                    elif part.rstrip(b'\x00') == b'file_not_exists':
                        if self.verbose or self.debug:
                            dc.stay()
                            print(f'File not exists on server: {path_from}')
                        os.remove(path_to)
                        return False
                dec_part = self.aes.unpad(self.aes.decrypt_part(part))
                ov_sign.update_hash(dec_part)
                f.write(dec_part)

    async def send_file(self, path_from, path_to=None):
        if not path_to:
            path_to = path_from
        await self.check_connection()
        self._validate_input('file', path_from)
        self.aes = ov_aes_cipher.AESCipher(key=oe_common.get_rnd_string(60))
        with open(path_from, 'rb') as f:
            file_size = os.path.getsize(path_from)
            '''if file_size > 1024*1024*500:  # 500 Mb
                signed = False
                sign = None
            else:
                signed = True
                sign = rsa.sign(f.read(), self.cr.private_key, 'SHA-1')
            f.seek(0)'''
            signed = True
            #self.reader, self.writer = await asyncio.open_connection(
                #self.server_address,
                #self.server_port
            #)
            self.writer.write(self._get_ov_header(
                'file',
                filename=path_to,
                key=rsa.encrypt(self.aes.key, self.server_public_key),
                iv=rsa.encrypt(self.aes.iv, self.server_public_key),
                signed=signed,
                # sign=sign,
                size=file_size
            ))
            # chunk_num = 0

            dc = oe_common.DinConsole()
            sc = oe_common.SpeedChecker('bits / second', 4096)
            ov_sign = ovcrypt.OvSign(self.cr.private_key)
            prefix_pad = b'\x00' * 3
            tx_bytes = 0
            file_size_str = oe_common.convert_size(file_size)
            for chunk in iter(lambda: f.read(4096), b''):
                # chunk_num += 1
                ov_sign.update_hash(chunk)
                if f.tell() >= file_size:
                    # print(f'\npadding chunk {chunk_num}')
                    chunk = self.aes.pad(chunk)
                    # prefix_pad = b'pad'
                    prefix_pad = True
                else:
                    # prefix_pad = b'\x00' * len(prefix_pad)
                    prefix_pad = False
                # self.writer.write(b'%d%b\n' % (len(chunk), prefix_pad))
                # self.writer.write(aes.encrypt_part(chunk))
                await self.write_with_prefix(self.aes.encrypt_part(chunk), prefix=prefix_pad)
                # await self.writer.drain()
                '''if chunk_num % 1000 == 0:
                    c_time_new = time.perf_counter()
                    c_diff = c_time_new - c_time
                    c_time = c_time_new
                    network_speed = 4096 * 1000 / c_diff * 8'''
                tx_bytes = tx_bytes + len(chunk)
                if self.debug or self.verbose:
                    dc.update('  Sent %s / %s, %s%%, speed: %s' % (
                        oe_common.convert_size(tx_bytes),
                        file_size_str,
                        round(tx_bytes / file_size * 100),
                        sc.get_speed()
                    ))
            if self.debug or self.verbose:
                dc.stay()
            await self.writer.drain()
            if self.debug:
                print("Data sent")
            if signed:
                sign = ov_sign.get_sign()
                prefix = b'\x00' * 3
                self.writer.write(b'%d%b\n' % (len(sign), prefix))
                sign_message = b'ovsign'
                padded_sign_message = sign_message + b'\x00' * (256 - len(sign_message))
                self.writer.write(padded_sign_message)
                self.writer.write(sign)
                await self.writer.drain()
                if self.debug:
                    print("Sign sent")
            # self.writer.write_eof()

            rcv_data = oe_common.fix_block_encoding_errors(self.aes.decrypt((await self.read_with_prefix())[0]))
            if self.debug or self.verbose:
                print(f'Answer: {rcv_data}')
            # self.writer.close()
            # await self.writer.wait_closed()
            return rcv_data

    def send_message_sync(self, message, timeout=2, retries=0):
        return self.loop.run_until_complete(
            self.send_message(message, timeout, retries)
        )

    async def send_message(self, message, timeout=2, retries=0):
        await self.check_connection()
        try:
            rcv = await self.send_data(self.server_address, data=message.encode(), timeout=timeout, retries=retries)
        except ConnectionResetError:
            if not retries:
                raise
            print(f'Connection error, {retries} left')
            await self.reconnect()
            return await self.send_message(message, timeout=timeout, retries=retries-1)
        if rcv == 'Access denied':
            self.server_public_key = None
            self.authorized = False
            return await self.send_message(message, timeout=timeout, retries=retries)
        elif not rcv:
            if self.verbose or self.debug:
                print(f'Reconnecting to {self.server_address}...')
            # await self.close_connection()
            return await self.send_message(message, timeout=timeout, retries=retries)
        return rcv

    async def send_data(self, address, data=b'', data_type='message', timeout=2, retries=0):
        if not self.connected:
            await self.connect()
        #self.reader, self.writer = await asyncio.open_connection(
            #address, server_port
        #)
        # sa = []
        # self._validate_input(data_type, filename)
        rnd_key = oe_common.get_rnd_string(60)
        self.aes = ov_aes_cipher.AESCipher(rnd_key)
        sign = None
        data_hash = None
        if data_type == 'public_key':
            header = self._get_ov_header(data_type)
        else:
            data_hash = rsa.compute_hash(data, 'SHA-256')
            sign = rsa.sign(data_hash, self.cr.private_key, 'SHA-1')
            data = self.aes.encrypt(data)
            header = self._get_ov_header(
                data_type,
                key=rsa.encrypt(self.aes.key, self.server_public_key),
                iv=rsa.encrypt(self.aes.iv, self.server_public_key),
                signed=True
            )
        self.writer.write(header)
        # print(f'header: {header}')
        prefix_pad = b'\x00' * 3  # data is padded, but now prefix is doesn't matter
        self.writer.write(b'%d%b\n' % (len(data), prefix_pad))
        if self.verbose:
            print(f'sending {oe_common.convert_size(len(data))}')
        self.writer.write(data)
        # await self.writer.drain()
        if sign:
            w = b'%d%b\n' % (len(sign), prefix_pad)
            self.writer.write(w)
            if self.debug:
                print('Sended: %s' % w)
            sign_message = b'ovsign'
            padded_sign_message = sign_message + b'\x00' * (256 - len(sign_message))
            self.writer.write(padded_sign_message)
            self.writer.write(sign)
            if self.debug:
                print('Sended: %s' % padded_sign_message)
                print('Sended sign (%sB): %s' % (len(sign), sign))
                print(f'Hash: {data_hash}')
        # self.writer.write_eof()
        await self.writer.drain()
        # print('Data sent!')
        # print(f'Header: "{header}"')
        if self.debug:
            print(f'Data: "{data}"')
            try:
                print(f'Data aes: "{self.aes.decrypt(data)}"')
            except:
                print('not aes')
        # rcv_data = await self.reader.read(4096)
        rcv_data = (await self.read_with_prefix(timeout=timeout, retries=retries))[0]
        # print(f'Encoded: {rcv_data}, len: {len(base64.b64decode(rcv_data))}')
        if data_type not in ['public_key', 'auth_req'] and self.server_public_key:
            # print(f'Decrypted: {aes.decrypt(rcv_data)}')
            if len(rcv_data) > 0:
                rcv_data = self.aes.decrypt(rcv_data)
                if data_type == 'message' and rcv_data == b'Sign ok':
                    sign_data = rcv_data  # Xz why
                    rcv_data = self.aes.decrypt((await self.read_with_prefix(timeout=timeout, retries=retries))[0])
                elif rcv_data == b'Sign error':
                    rcv_data = json.dumps({'status': False, 'description': 'Sign error'}).encode()
                rcv_data = oe_common.fix_block_encoding_errors(rcv_data)
            else:
                await self.close_connection()
                return False
            # if self.debug or self.verbose:
            #    print(f'Answer: {rcv_data}')
        # self.writer.close()
        # await self.writer.wait_closed()
        return rcv_data

    async def manual_send(self, mode):
        while True:
            try:
                if mode in ['uf']:
                    m = input('Input file name: ')
                    if not os.path.isfile(m):
                        m = os.path.join(pathlib.Path().absolute(), m)
                        if not os.path.isfile(m):
                            print(f'File not exist: {m}')
                            continue
                    to = input('Path on server: ')
                    # return await self.send_file(m)
                    print(await self.send_file(m, to))
                elif mode == 'df':
                    m = input('File path on server: ')
                    to = input('Local path: ')
                    print(await self.get_file(m, to))
                elif mode == 'vfs':
                    m = input('File path on server: ')
                    print(await self.verify_file_sign(m))
                else:
                    m = input('Enter message: ')
                    # return await self.send_message(m)
                    if m == 'c':
                        await self.close_connection()
                    else:
                        print(await self.send_message(m))
            except ConnectionRefusedError as e:
                print(f"Connection refused: {e}")
                # return False



import sys
from ovtp.client import run_client
from ovtp.server import run_server


if len(sys.argv) < 2 or sys.argv[1] in ['-h', '--help']:
    print("Usage: python3 -m ovtp [client | server]")
    sys.exit(0)

if sys.argv[1] == 'client':
    run_client()
elif sys.argv[1] == 'server':
    run_server()
else:
    print("Unknown mode")
    sys.exit(1)



import json
import os
import rsa
import pathlib
import ovcrypt
import asyncio
import ov_aes_cipher
import oe_common
from ovtp.server import cfg, short_key_to_full, get_short_rsa_key


class OvtpServer:
    class Handler:
        def __init__(self, server, reader, writer):
            self.server = server
            self.reader = reader
            self.writer = writer
            self.new_message_timeout = 30
            self.aes = None
            self.cr = ovcrypt.OvCrypt()

        def check_key_is_master(self, key):
            auth_keys_path = os.path.join(cfg['auth_keys_dir'], 'authorized_keys')
            os.makedirs(cfg['auth_keys_dir'], exist_ok=True)
            # oe_common.check_create_dir(auth_keys_path)
            saved_master_keys = []
            if os.path.isfile(auth_keys_path):
                with open(auth_keys_path, 'rb') as f:
                    for line in f:
                        if self.server.debug:
                            print(f'found saved rsa key: {line}')
                        saved_master_keys.append(rsa.PublicKey.load_pkcs1(short_key_to_full(line)))
            if key not in saved_master_keys:
                master_keys = self.cr.get_master_keys()
                for m_key in master_keys:
                    if m_key not in saved_master_keys:
                        saved_master_keys.append(m_key)
                with open(auth_keys_path, 'wb') as f:
                    f.write(b'\n'.join([get_short_rsa_key(rsa.PublicKey.save_pkcs1(k).replace(b'\n', b'')) for k in
                                        saved_master_keys]))
            if key in saved_master_keys:
                if self.server.debug:
                    print(f'Key found in master keys: {key}')
                return True, True
            if self.server.verbose:
                print('Key not found in master keys')
            return False, 'no_key'

        @staticmethod
        def unpad_ov_header(s):
            return bytes(s).rstrip(b'\x00')

        async def parse_header(self, read_bytes):
            length = int.from_bytes(await asyncio.wait_for(self.reader.read(read_bytes), timeout=30), byteorder='big')
            if length > 0:
                return await asyncio.wait_for(self.reader.read(length), timeout=30)
            else:
                return None

        async def receive_headers(self):
            # Waiting for self.headers_timeout, then set up it to standard 30 seconds
            # This needed to make long timeout between messages, without closing the connection
            data_type = self.unpad_ov_header(
                await asyncio.wait_for(self.reader.read(10), timeout=self.new_message_timeout)
            )
            self.new_message_timeout = 30
            if self.server.debug:
                print(f'Data type is {data_type}')

            filename_b = await self.parse_header(4)
            filename = oe_common.fix_block_encoding_errors(filename_b) if filename_b else ''
            header_key = await self.parse_header(4)
            header_iv = await self.parse_header(4)
            header_signed = True if await asyncio.wait_for(self.reader.read(1), timeout=5) else False  # y/n
            if data_type == b'file':
                header_filesize = int.from_bytes(await asyncio.wait_for(self.reader.read(14), timeout=30),
                                                 byteorder='big')
            else:
                header_filesize = None

            return data_type, filename, header_key, header_iv, header_signed, header_filesize

        async def read_with_prefix(self):
            prefix = (await asyncio.wait_for(self.reader.readline(), timeout=30)).rstrip()
            pk_data = b''
            if prefix != b'':
                msg_len = int(prefix[:-3])
                pk_data = await asyncio.wait_for(self.reader.readexactly(msg_len), timeout=5)
            return pk_data

        async def write_with_prefix(self, data, prefix=False):
            if prefix:
                prefix_pad = b'pad'
            else:
                prefix_pad = b'\x00' * 3
            self.writer.write(b'%d%b\n%b' % (len(data), prefix_pad, data))
            await self.writer.drain()

        async def close_connection(self, address):
            print(f'Closing connection with {self.str_ip(address)}')
            self.writer.write_eof()
            await self.writer.drain()
            self.writer.close()
            await self.writer.wait_closed()

        @staticmethod
        def str_ip(address):
            if isinstance(address, tuple) and len(address) > 1:
                return f'{address[0]}:{address[1]}'
            else:
                return 'Unknown'

        async def handle_packet(self):
            ov_sign = None
            address = self.writer.get_extra_info('peername')
            print(f"Connection from {self.str_ip(address)}")
            while True:
                try:
                    (data_type,
                     filename,
                     header_key,
                     header_iv,
                     header_signed,
                     header_filesize) = await self.receive_headers()
                except (asyncio.exceptions.TimeoutError, ConnectionResetError) as e:
                    print(f'{type(e).__name__}({e}) occurred')
                    await self.close_connection(address)
                    return False

                read_part_counter = 0
                received_len = 0

                data = b''
                data_parts = []
                # last_part = False

                if header_iv and header_key:
                    self.aes = ov_aes_cipher.AESCipher(
                        hash_key=rsa.decrypt(header_key, self.cr.private_key),
                        iv=rsa.decrypt(header_iv, self.cr.private_key)
                    )
                else:
                    self.aes = ov_aes_cipher.AESCipher('pass')

                if data_type == b'file' and os.path.isfile(filename):
                    os.remove(filename)

                if address in self.server.saved_keys:
                    ov_sign = ovcrypt.OvSign(self.server.saved_keys[address].key)
                part_data = b''
                msg_pad = b''
                dc = oe_common.DinConsole()
                sc = oe_common.SpeedChecker('bits / second', 4096)
                rx_bytes = 0
                file_size_str = oe_common.convert_size(header_filesize) if header_filesize else 'no size'

                if self.reader.at_eof():
                    print(f'Received eof from {self.str_ip(address)}')
                    await self.close_connection(address)
                    return False

                while True:
                    read_part_counter += 1

                    if data_type == b'':
                        print('Data type empty, closing...')
                        await self.close_connection(address)
                        return False
                    elif data_type == b'public_key':
                        # print('Request for public key')
                        prefix = (await asyncio.wait_for(self.reader.readline(), timeout=5)).rstrip()
                        # print(f'prefix: {prefix}')
                        pk_data = b''
                        if prefix != b'':
                            msg_len = int(prefix[:-3])
                            # print(f'Receiving {msg_len} bytes')
                            pk_data = await asyncio.wait_for(self.reader.readexactly(msg_len), timeout=5)
                        # print(f'received: {pk_data}')
                        self.server.saved_keys[address] = self.cr.Key(rsa.PublicKey.load_pkcs1(
                            pk_data))
                        # self.writer.write(rsa.PublicKey.save_pkcs1(self.cr.public_key))
                        await self.write_with_prefix(rsa.PublicKey.save_pkcs1(self.cr.public_key))
                        break
                    elif data_type == b'auth_req':
                        prefix = (await asyncio.wait_for(self.reader.readline(), timeout=5)).rstrip()
                        pk_data = b''
                        if prefix != b'':
                            msg_len = int(prefix[:-3])
                            # print(f'Receiving {msg_len} bytes')
                            pk_data = await asyncio.wait_for(self.reader.readexactly(msg_len), timeout=5)
                        if self.server.debug:
                            print(f'pk_data: {pk_data}')
                        v_string = oe_common.get_rnd_string(100).encode()
                        self.server.saved_keys[address].verification_string = v_string
                        # self.writer.write(rsa.encrypt(v_string, self.server.saved_keys[address].key))
                        await self.write_with_prefix(rsa.encrypt(v_string, self.server.saved_keys[address].key))
                        if self.server.debug:
                            print(f'generated string: {v_string}')
                        r = await self.read_with_prefix()
                        sign = await self.reader.read(256)
                        break
                    elif data_type == b'auth_resp':
                        if self.server.debug:
                            print(f'Received auth resp')
                    else:
                        if address in self.server.saved_keys and self.server.saved_keys[address].auth:
                            if self.server.debug:
                                print(f'Data type: {data_type}')
                                print('Key already authorized')
                            if data_type == b'get_file':
                                if self.server.debug:
                                    print(f'Get file request: {filename}')
                                if os.path.isfile(filename):
                                    ov_sign = ovcrypt.OvSign(self.cr.private_key)
                                    file_size = os.path.getsize(filename)
                                    tx_bytes = 0
                                    with open(filename, 'rb') as f:
                                        for chunk in iter(lambda: f.read(4096), b''):
                                            ov_sign.update_hash(chunk)
                                            await self.write_with_prefix(
                                                self.aes.encrypt_part(self.aes.pad(chunk)),
                                                prefix=True
                                            )
                                            # await self.writer.drain()
                                            tx_bytes = tx_bytes + len(chunk)
                                            if self.server.debug or self.server.verbose:
                                                dc.update('  Sent %s / %s, %s%%, speed: %s' % (
                                                    oe_common.convert_size(tx_bytes),
                                                    file_size_str,
                                                    round(tx_bytes / file_size * 100),
                                                    sc.get_speed()
                                                ))
                                    # sign_message = b'ovsign'
                                    # padded_sign_message = sign_message + b'\x00' * (256 - len(sign_message))
                                    # await self.write_with_prefix(padded_sign_message)
                                    await self.write_with_prefix(self.cr.ov_pad(b'ov_sign', 256))
                                    self.writer.write(ov_sign.get_sign())
                                    await self.writer.drain()
                                else:
                                    await self.write_with_prefix(self.cr.ov_pad(b'file_not_exists', 256))
                                    # await self.write_with_prefix(self.cr.ov_pad(b'transmission_ended', 256))
                                    # raise RuntimeError(f'Requested from {address} file not exists: {filename}')
                                break
                            elif data_type == b'get_hash':
                                if self.server.debug:
                                    print(f'Get file hash request: {filename}')
                                if os.path.isfile(filename):
                                    ov_sign = ovcrypt.OvSign()  # key is not needed, we don't get sign, only hash
                                    with open(filename, 'rb') as f:
                                        for chunk in iter(lambda: f.read(4096), b''):
                                            ov_sign.update_hash(chunk)
                                        # await self.write_with_prefix(self.aes.encrypt(ov_sign.get_hash()))
                                        await self.write_with_prefix(rsa.encrypt(
                                            ov_sign.get_hash(),
                                            self.server.saved_keys[address].key
                                        ))
                                else:
                                    await self.write_with_prefix(rsa.encrypt(
                                        b'file_not_exists',
                                        self.server.saved_keys[address].key
                                    ))
                                    # raise RuntimeError(f'Requested from {address} file not exists: {filename}')
                                break
                        else:
                            # self.writer.write(aes.encrypt('Access denied'.encode()))
                            # self.writer.drain()
                            await self.write_with_prefix(self.aes.encrypt('Access denied'.encode()))
                            self.writer.close()
                            await self.writer.wait_closed()
                            raise RuntimeError('Error, key not authorized!')

                    try:
                        # part_data = await self.reader.readexactly(4096)
                        prefix = (await asyncio.wait_for(self.reader.readline(), timeout=5)).rstrip()
                        # print(f'prefix: {prefix}')
                        if prefix != b'':
                            msg_pad = prefix[-3:]
                            msg_len = int(prefix[:-3])
                            # print(f'Receiving {msg_len} bytes')
                            part_data = await asyncio.wait_for(self.reader.readexactly(msg_len), timeout=5)
                            if self.server.debug:
                                print(f'Received: {part_data}')
                    except asyncio.exceptions.IncompleteReadError as e:
                        # part_data = e.partial
                        # last_part = True
                        raise RuntimeError("incomplete read is not supported")

                    if len(part_data) > 0:
                        # data += part_data
                        if len(part_data) == 256:
                            # print('len is 256, breaking')
                            if part_data.rstrip(b'\x00') == b'ovsign':
                                received_sign = await asyncio.wait_for(self.reader.readexactly(256), timeout=5)
                                if self.server.debug:
                                    print('ovsign received (%sB): %s' % (len(received_sign), received_sign))
                                if data_type != b'auth_resp':
                                    if ov_sign.get_verification_result(received_sign):
                                        # self.writer.write(aes.encrypt(b'File sign ok'))
                                        if self.server.debug:
                                            print(f'Sign ok, expected {ov_sign.get_hash()}')
                                        await self.write_with_prefix(self.aes.encrypt(b'Sign ok'))
                                    else:
                                        # self.writer.write(aes.encrypt(b'File sign error'))
                                        if self.server.debug:
                                            print(f'Sign error, expected {ov_sign.get_hash()}')
                                        await self.write_with_prefix(self.aes.encrypt(b'Sign error'))
                                        return
                                break
                        if data_type == b'file':
                            if filename[0] != '/':
                                filename = os.path.join(pathlib.Path().absolute(), filename)
                            oe_common.check_create_dir(filename)
                            part_data_decoded = self.aes.decrypt_part(part_data)
                            if msg_pad == b'pad':
                                part_data_decoded = self.aes.unpad(part_data_decoded)
                            ov_sign.update_hash(part_data_decoded)
                            with open(filename, 'ab') as f:
                                f.write(part_data_decoded)
                                # print(f'part {read_part_counter} writed')
                        else:
                            part_data_decoded = self.aes.decrypt(part_data)
                            if msg_pad == b'pad':
                                part_data_decoded = self.aes.unpad(part_data_decoded)
                            if self.server.debug:
                                print(f'Decoded part data: {part_data_decoded}')
                            data_parts.append(part_data_decoded)
                            ov_sign.update_hash(part_data_decoded)
                        received_len += len(part_data)

                        if received_len and header_filesize:
                            rx_bytes = rx_bytes + len(part_data)
                            if self.server.debug or self.server.verbose:
                                dc.update('  Received %s / %s, %s%%, speed: %s' % (
                                    oe_common.convert_size(rx_bytes),
                                    file_size_str,
                                    round(rx_bytes / (header_filesize if header_filesize else 0) * 100),
                                    sc.get_speed()
                                ))
                        else:
                            if self.server.debug or self.server.verbose:
                                print(f'  Received {read_part_counter}')
                if self.server.debug or self.server.verbose:
                    dc.clear()
                if self.server.debug or self.server.verbose:
                    print(f'Received {oe_common.convert_size(received_len)} from {address}')
                if data_type == b'message':
                    data = b''.join(data_parts)
                    del data_parts
                    dec_data = oe_common.fix_block_encoding_errors(data)
                    if self.server.debug or self.server.verbose:
                        print('message:', dec_data)
                    status, description = self.server.callback(True, dec_data)
                    d = json.dumps({'status': status, 'description': description}).encode()
                    await self.write_with_prefix(self.aes.encrypt(d))
                    self.new_message_timeout = cfg['new_message_timeout'] * 60
                elif data_type == b'auth_resp':
                    data = b''.join(data_parts)
                    del data_parts
                    if self.server.debug:
                        print(f'auth resp is: {data}')
                    if data == self.server.saved_keys[address].verification_string:
                        status, response = self.check_key_is_master(self.server.saved_keys[address].key)
                        if status:
                            # self.writer.write(aes.encrypt('auth successful'.encode()))
                            await self.write_with_prefix(self.aes.encrypt('auth successful'.encode()))
                            self.server.saved_keys[address].auth = True
                        else:
                            if self.server.verbose:
                                print(f'Auth failed: {response}')
                            # self.writer.write(self.aes.encrypt('auth failed 1'.encode()))
                            await self.write_with_prefix(self.aes.encrypt('auth failed 1'.encode()))
                    else:
                        if self.server.verbose:
                            print(f'Verification string is not valid')
                        # self.writer.write(self.aes.encrypt('auth failed 2'.encode()))
                        await self.write_with_prefix(self.aes.encrypt('auth failed 2'.encode()))
                await self.writer.drain()
                if self.server.debug:
                    print(f'End of packet {address}')

    def __init__(self, callback, verbose=None, debug=None):
        self.server = None
        self.callback = callback
        self.saved_keys = {}
        self.verbose = verbose
        self.debug = debug

    async def run_packet_handler(self, reader, writer):
        handler = self.Handler(self, reader, writer)
        await handler.handle_packet()

    def get_sockets(self):
        return ', '.join(str(sock.getsockname()) for sock in self.server.sockets)

    async def run_server(self):
        self.server = await asyncio.start_server(
            self.run_packet_handler, cfg['server_ip'], cfg['server_port']
        )
        # print(f'Serving on {self.get_sockets}')
        async with self.server:
            await self.server.serve_forever()



import signal
import datetime
import asyncio
import os
from ovtp.client import OvtpClient


def signal_handler(sig, frame):
    print("Pressed Ctrl+C, exiting...")
    os.kill(os.getpid(), signal.SIGTERM)


def run_client():
    print("Running script on %s" % datetime.datetime.now())
    signal.signal(signal.SIGINT, signal_handler)
    print('Press Ctrl+C for exit')
    server_ip = input('Server ip: ')
    oec = OvtpClient(server_ip, verbose=False, debug=False)
    input_text = 'Select mode (m - message, df - download file, uf - upload file, vfs - verify file sign)'
    main_mode = input(input_text + ' or d, v, dv - debug, verbose: ')
    if main_mode in ['d', 'v', 'dv']:
        if main_mode in ['d', 'dv']:
            oec.debug = True
        if main_mode in ['v', 'dv']:
            oec.verbose = True
        main_mode = input(f'{input_text}: ')
        if main_mode not in ['m', 'df', 'uf', 'vfs']:
            raise ValueError(f"Incorrect mode: {main_mode}")
    asyncio.run(oec.manual_send(main_mode))



import ovcfg
import os


if os.name == 'posix':
    config_dirs = [
        [['/', 'var', 'lib'], []],
        [[os.path.expanduser("~"), '.local'], ['share']]
    ]

    for config_dir in config_dirs:
        if os.access(os.path.join(*config_dir[0]), os.W_OK):
            config_path = os.path.join(*config_dir[0], *config_dir[1], 'ovtp')
            break
    else:
        raise RuntimeError("Can't create ovcrypt config directory")
elif os.name == 'nt':
    config_path = os.path.join(os.getenv('APPDATA'), 'ovtp')
else:
    raise RuntimeError(f'Unknown platform: {os.name}')


sc = {
    'server_ip': '0.0.0.0',
    'server_port': 888,
    'auth_keys_dir': os.path.join(config_path, 'auth_keys'),
    'new_message_timeout': 15,  # 15 min default
}
cfg = ovcfg.Config(std_config=sc, file='server.json', cfg_dir_name='ovtp').import_config()



import os
from ovtp.server import get_short_rsa_key, cfg
from pathlib import Path


def add_auth_keys():
    auth_keys_path = os.path.join(cfg['auth_keys_dir'], 'authorized_keys')
    if not os.path.isfile(auth_keys_path):
        Path(auth_keys_path).touch()
    if len(ls_dir := os.listdir(cfg['auth_keys_dir'])) > 1:
        with open(auth_keys_path, 'rb+') as f:
            keys = f.read().splitlines()
            for new_key_file in ls_dir:
                if new_key_file != 'authorized_keys':
                    with open(os.path.join(cfg['auth_keys_dir'], new_key_file), 'rb') as f2:
                        new_key = get_short_rsa_key(f2.read().replace(b'\n', b''))
                        key_name = f'{new_key_file} {new_key[:10].decode()}...{new_key[-10:].decode()}'
                        if new_key in keys:
                            print(f"Key {key_name} already added")
                        else:
                            print(f"Added key: {key_name}")
                            f.write(new_key + b'\n')
    else:
        print(f"Keys not found in directory: {cfg['auth_keys_dir']}")



import re


def get_short_rsa_key(key):
    r = re.search(rb'-----BEGIN RSA PUBLIC KEY-----(.*)-----END RSA PUBLIC KEY-----', key)
    if r:
        return r.group(1)
    else:
        raise ValueError(f'Bad rsa key: {key}')


def short_key_to_full(key):
    return b'%b\n%b\n%b' % (
        b'-----BEGIN RSA PUBLIC KEY-----',
        key,
        b'-----END RSA PUBLIC KEY-----'
    )
